from .user import *  # noqa
from .tweet import *  # noqa
from .media import *  # noqa
from .poll import *  # noqa
from .place import *  # noqa
from .compliance import *  # noqa
from .space import *  # noqa
from .list import *  # noqa
from .stream import *  # noqa
from .ext import *  # noqa



"""
    extends objects

    Refer: https://developer.twitter.com/en/docs/twitter-api/expansions
"""
from dataclasses import dataclass, field
from typing import List, Optional, Union

from . import (
    BaseModel,
    Media,
    Place,
    Poll,
    Tweet,
    User,
    Space,
    Topic,
    ComplianceJob,
    StreamRule,
    TwitterList,
)


@dataclass
class Error(BaseModel):
    """
    A class representing the error object for request.
    """

    detail: Optional[str] = field(default=None)
    title: Optional[str] = field(default=None)
    resource_type: Optional[str] = field(default=None, repr=False)
    parameter: Optional[str] = field(default=None, repr=False)
    value: Optional[str] = field(default=None, repr=False)
    type: Optional[str] = field(default=None, repr=False)


@dataclass
class MetaSummary(BaseModel):
    """
    A class representing the meta summary object for request.
    Now works for stream rules api.
    """

    created: Optional[int] = field(default=None)
    not_created: Optional[int] = field(default=None, repr=False)
    valid: Optional[int] = field(default=None)
    invalid: Optional[int] = field(default=None, repr=False)
    deleted: Optional[int] = field(default=None)
    not_deleted: Optional[int] = field(default=None, repr=False)


@dataclass
class Meta(BaseModel):
    """
    A class representing the meta object for request.
    """

    result_count: Optional[int] = field(default=None)
    previous_token: Optional[str] = field(default=None, repr=False)
    next_token: Optional[str] = field(default=None, repr=False)
    oldest_id: Optional[str] = field(default=None, repr=False)
    newest_id: Optional[str] = field(default=None, repr=False)
    sent: Optional[str] = field(default=None, repr=False)
    summary: Optional[MetaSummary] = field(default=None, repr=False)
    total_tweet_count: Optional[int] = field(default=None, repr=False)


@dataclass
class Includes(BaseModel):
    """
    A class representing the expansions object for main request thread.
    """

    media: Optional[List[Media]] = field(default=None, compare=False)
    places: Optional[List[Place]] = field(default=None, compare=False)
    polls: Optional[List[Poll]] = field(default=None, compare=False)
    tweets: Optional[List[Tweet]] = field(default=None, compare=False)
    users: Optional[List[User]] = field(default=None, compare=False)
    topics: Optional[List[Topic]] = field(default=None, compare=False)


@dataclass
class TweetCount(BaseModel):
    """
    A class representing the counts for tweets match a search query.
    Refer: https://developer.twitter.com/en/docs/twitter-api/tweets/counts/api-reference/get-tweets-counts-recent
    """

    start: Optional[str] = field(default=None)
    end: Optional[str] = field(default=None)
    tweet_count: Optional[int] = field(default=None)


@dataclass
class Response:
    """
    A class representing for the twitter response.
    """

    data: Optional[
        Union[
            User,
            Tweet,
            Media,
            Poll,
            Place,
            Space,
            ComplianceJob,
            TwitterList,
            List[User],
            List[Tweet],
            List[Media],
            List[Poll],
            List[Place],
            List[StreamRule],
            List[TweetCount],
            List[TwitterList],
            List[Space],
            List[ComplianceJob],
        ]
    ]
    includes: Optional[Includes] = field(default=None, repr=False)
    meta: Optional[Meta] = field(default=None, repr=False)
    errors: Optional[List[Error]] = field(default=None, repr=False)



"""
    user object

    Refer: https://developer.twitter.com/en/docs/twitter-api/data-dictionary/object-model/user
"""

from dataclasses import dataclass, field
from typing import List, Optional

from .base import BaseModel


@dataclass
class UserEntitiesUrlObj(BaseModel):
    start: Optional[int] = field(default=None, repr=False)
    end: Optional[int] = field(default=None, repr=False)
    url: Optional[str] = field(default=None)
    expanded_url: Optional[str] = field(default=None, repr=False)
    display_url: Optional[str] = field(default=None, repr=False)


@dataclass
class UserEntitiesHashtag(BaseModel):
    start: Optional[int] = field(default=None, repr=False)
    end: Optional[int] = field(default=None, repr=False)
    tag: Optional[str] = field(default=None)


@dataclass
class UserEntitiesMention(UserEntitiesHashtag):
    ...


@dataclass
class UserEntitiesDescription(BaseModel):
    urls: Optional[List[UserEntitiesUrlObj]] = field(default=None, repr=False)
    hashtags: Optional[List[UserEntitiesHashtag]] = field(default=None, repr=False)
    mentions: Optional[List[UserEntitiesMention]] = field(default=None, repr=False)
    cashtags: Optional[List[UserEntitiesHashtag]] = field(default=None, repr=False)


@dataclass
class UserEntitiesUrl(BaseModel):
    urls: Optional[List[UserEntitiesUrlObj]] = field(default=None)


@dataclass
class UserEntities(BaseModel):
    url: Optional[UserEntitiesUrl] = field(default=None)
    description: Optional[UserEntitiesDescription] = field(default=None, repr=False)


@dataclass
class PublicMetrics(BaseModel):
    followers_count: Optional[int] = field(default=None)
    following_count: Optional[int] = field(default=None)
    tweet_count: Optional[int] = field(default=None, repr=False)
    listed_count: Optional[int] = field(default=None, repr=False)


@dataclass
class UserWithheld(BaseModel):
    """
    refer: https://help.twitter.com/en/rules-and-policies/tweet-withheld-by-country
    """

    scope: Optional[str] = field(default=None)
    country_codes: Optional[List[str]] = field(default=None, repr=False)


@dataclass
class User(BaseModel):
    """
    A class representing the user object.
    """

    id: Optional[str] = field(default=None)
    name: Optional[str] = field(default=None, compare=False)
    username: Optional[str] = field(default=None, compare=False)
    created_at: Optional[str] = field(default=None, repr=False, compare=False)
    description: Optional[str] = field(default=None, repr=False, compare=False)
    location: Optional[str] = field(default=None, repr=False, compare=False)
    pinned_tweet_id: Optional[str] = field(default=None, repr=False, compare=False)
    profile_image_url: Optional[str] = field(default=None, repr=False, compare=False)
    protected: Optional[bool] = field(default=None, repr=False, compare=False)
    url: Optional[str] = field(default=None, repr=False, compare=False)
    verified: Optional[bool] = field(default=None, repr=False, compare=False)
    entities: Optional[UserEntities] = field(default=None, repr=False, compare=False)
    public_metrics: Optional[PublicMetrics] = field(
        default=None, repr=False, compare=False
    )
    withheld: Optional[UserWithheld] = field(default=None, repr=False, compare=False)



"""
    Model for list.

    Refer: https://developer.twitter.com/en/docs/twitter-api/lists/manage-lists/api-reference/post-lists
"""

from dataclasses import dataclass, field
from typing import Optional

from .base import BaseModel


@dataclass
class TwitterList(BaseModel):
    id: Optional[str] = field(default=None)
    name: Optional[str] = field(default=None)
    created_at: Optional[str] = field(default=None, repr=False)
    description: Optional[str] = field(default=None, repr=False)
    follower_count: Optional[int] = field(default=None, repr=False)
    member_count: Optional[int] = field(default=None, repr=False)
    private: Optional[bool] = field(default=None, repr=False)
    owner_id: Optional[str] = field(default=None, repr=False)



"""
    Parameters validators
"""
from typing import List, Tuple, Union

from pytwitter.error import PyTwitterError


def enf_comma_separated(name: str, value: Union[str, List, Tuple]):
    """
    Check to see if field's value type belong to correct type.
    If accepted, return formatted value, otherwise, raise a Error.
    :param name: The parameter field name.
    :param value: Value for provide.
    :return: Formatted comma-separated list
    """

    if not value:
        return None

    if isinstance(value, str):
        return value
    elif isinstance(value, (List, Tuple)):
        return ",".join(value)
    else:
        raise PyTwitterError(
            f"Parameter {name} must be comma-separated str, list or tuple"
        )



"""
    Space object.

    Refer: https://developer.twitter.com/en/docs/twitter-api/data-dictionary/object-model/space
"""

from dataclasses import dataclass, field
from typing import List, Optional

from .base import BaseModel


@dataclass
class Space(BaseModel):
    """
    A class representing the space object.
    """

    id: Optional[str] = field(default=None)
    state: Optional[str] = field(default=None)
    created_at: Optional[str] = field(default=None, repr=False)
    ended_at: Optional[str] = field(default=None, repr=False)
    host_ids: Optional[List[str]] = field(default=None, repr=False)
    lang: Optional[str] = field(default=None, repr=False)
    is_ticketed: Optional[bool] = field(default=None, repr=False)
    invited_user_ids: Optional[List[str]] = field(default=None, repr=False)
    participant_count: Optional[int] = field(default=None, repr=False)
    scheduled_start: Optional[str] = field(default=None, repr=False)
    speaker_ids: Optional[List[str]] = field(default=None, repr=False)
    started_at: Optional[str] = field(default=None, repr=False)
    title: Optional[str] = field(default=None, repr=False)
    topic_ids: Optional[List[str]] = field(default=None, repr=False)
    updated_at: Optional[str] = field(default=None, repr=False)


@dataclass
class Topic(BaseModel):
    """
    A class representing the space topic object.
    """

    id: Optional[str] = field(default=None)
    name: Optional[str] = field(default=None)
    description: Optional[str] = field(default=None, repr=False)



"""
    Media object

    Refer: https://developer.twitter.com/en/docs/twitter-api/data-dictionary/object-model/media
"""

from dataclasses import dataclass, field
from typing import Optional

from .base import BaseModel


@dataclass
class MediaNonPublicMetrics(BaseModel):
    playback_0_count: Optional[int] = field(default=None)
    playback_100_count: Optional[int] = field(default=None, repr=False)
    playback_25_count: Optional[int] = field(default=None, repr=False)
    playback_50_count: Optional[int] = field(default=None, repr=False)
    playback_75_count: Optional[int] = field(default=None, repr=False)


@dataclass
class MediaOrganicMetrics(MediaNonPublicMetrics):
    view_count: Optional[int] = field(default=None, repr=False)


@dataclass
class MediaPromotedMetrics(MediaOrganicMetrics):
    ...


@dataclass
class MediaPublicMetrics(BaseModel):
    view_count: Optional[int] = field(default=None, repr=False)


@dataclass
class Media(BaseModel):
    """
    A class representing the media object.
    """

    media_key: Optional[str] = field(default=None)
    type: Optional[str] = field(default=None)
    url: Optional[str] = field(default=None, repr=False)
    duration_ms: Optional[int] = field(default=None, repr=False)
    height: Optional[int] = field(default=None, repr=False)
    non_public_metrics: Optional[MediaNonPublicMetrics] = field(
        default=None, repr=False
    )
    organic_metrics: Optional[MediaOrganicMetrics] = field(default=None, repr=False)
    preview_image_url: Optional[str] = field(default=None, repr=False)
    promoted_metrics: Optional[MediaPromotedMetrics] = field(default=None, repr=False)
    public_metrics: Optional[MediaPublicMetrics] = field(default=None, repr=False)
    width: Optional[int] = field(default=None, repr=False)
    alt_text: Optional[str] = field(default=None, repr=False)



"""
    Parameter convertors
"""
from pytwitter.error import PyTwitterError


def conv_type(field, _type, value):
    """
    Convert value to target type, and Checked.
    :param field: Name of the field you are checking.
    :param _type: Type that the value should be returned as.
    :param value: Value to convert to _type.
    :return:
    """
    try:
        return _type(value)
    except (ValueError, TypeError):
        raise PyTwitterError(f'"{field}" must be type {_type.__name__}')
